@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: parserintf.h,v 1.15 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -19,6 +19,9 @@
 #define PARSERINTF_H
 
 #include <qdict.h>
+#include <qstrlist.h>
+
+#include "types.h"
 
 class Entry;
 class FileDef;
@@ -36,15 +39,37 @@ class ParserInterface
 {
   public:
     virtual ~ParserInterface() {}
+
+    /** Starts processing a translation unit (source files + headers).
+     *  After this call parseInput() is called with sameTranslationUnit
+     *  set to FALSE. If parseInput() returns additional include files,
+     *  these are also processed using parseInput() with 
+     *  sameTranslationUnit set to TRUE. After that
+     *  finishTranslationUnit() is called.
+     */
+    virtual void startTranslationUnit(const char *fileName) = 0;
+
+    /** Called after all files in a translation unit have been 
+     *  processed.
+     */
+    virtual void finishTranslationUnit() = 0;
+
     /** Parses a single input file with the goal to build an Entry tree. 
      *  @param[in] fileName    The full name of the file.
      *  @param[in] fileBuf     The contents of the file (zero terminated).
      *  @param[in,out] root    The root of the tree of Entry *nodes 
      *             representing the information extracted from the file.
+     *  @param[in] sameTranslationUnit TRUE if this file was found in the same
+     *             translation unit (in the filesInSameTranslationUnit list
+     *             returned for another file).
+     *  @param[in,out] filesInSameTranslationUnit other files expected to be
+     *              found in the same translation unit (used for libclang)
      */
     virtual void parseInput(const char *fileName,
                             const char *fileBuf,
-                            Entry *root) = 0;
+                            Entry *root,
+                            bool sameTranslationUnit,
+                            QStrList &filesInSameTranslationUnit) = 0;
 
     /** Returns TRUE if the language identified by \a extension needs
      *  the C preprocessor to be run before feed the result to the input
@@ -56,6 +81,7 @@ class ParserInterface
     /** Parses a source file or fragment with the goal to produce
      *  highlighted and cross-referenced output.
      *  @param[in] codeOutIntf Abstract interface for writing the result.
+     *  @param[in] lang The programming language of the code fragment.
      *  @param[in] scopeName Name of scope to which the code belongs.
      *  @param[in] input Actual code in the form of a string
      *  @param[in] isExampleBlock TRUE iff the code is part of an example.
@@ -72,10 +98,12 @@ class ParserInterface
      *  @param[in] showLineNumbers if set to TRUE and also fileDef is not 0,
      *             line numbers will be added to the source fragement
      *  @param[in] searchCtx context under which search data has to be stored.
+     *  @param[in] collectXRefs collect cross-reference relations.
      */
     virtual void parseCode(CodeOutputInterface &codeOutIntf,
                            const char *scopeName,
                            const QCString &input,
+                           SrcLangExt lang,
                            bool isExampleBlock,
                            const char *exampleName=0,
                            FileDef *fileDef=0,
@@ -84,7 +112,8 @@ class ParserInterface
                            bool inlineFragment=FALSE,
                            MemberDef *memberDef=0,
                            bool showLineNumbers=TRUE,
-                           Definition *searchCtx=0
+                           Definition *searchCtx=0,
+                           bool collectXRefs=TRUE
                           ) = 0;
 
     /** Resets the state of the code parser.
@@ -155,8 +184,8 @@ class ParserManager
      */
     ParserInterface *getParser(const char *extension)
     {
-      if (extension==0) return m_defaultParser;
       QCString ext = QCString(extension).lower();
+      if (ext.isEmpty()) ext=".no_extension";
       ParserInterface *intf = m_extensions.find(ext);
       if (intf==0 && ext.length()>4)
       {
